#
#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Cambodia classes (implemented as groups+sections)
"""

from zope.component import adapts
from zope.container.interfaces import INameChooser
from zope.intid import addIntIdSubscriber
from zope.lifecycleevent.interfaces import IObjectAddedEvent
from zope.lifecycleevent.interfaces import IObjectRemovedEvent
from zope.lifecycleevent import ObjectAddedEvent
from zope.event import notify

from schooltool.relationship.interfaces import IRelationshipAddedEvent
from schooltool.relationship.interfaces import IRelationshipRemovedEvent
from schooltool.relationship import URIObject, RelationshipSchema
from schooltool.relationship import RelationshipProperty
from schooltool.relationship import getRelatedObjects
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.schoolyear.subscriber import ObjectEventAdapterSubscriber
from schooltool.term.interfaces import ITerm
from schooltool.group.interfaces import IGroupContainer
from schooltool.group.group import Group
from schooltool.group.group import URIMembership, URIMember, URIGroup
from schooltool.course.interfaces import ISectionContainer
from schooltool.course.section import Section
from schooltool.cambodia.grade import interfaces
from schooltool.cambodia.grade.level import URILevel, URICourse, URILevelCourses



URILevelGroups = URIObject(
    'http://schooltool.org/ns/cambodia/levelgroups/',
    'Groups at the Level.',
    'Groups taught at this level this year.')

URILevelGroup = URIObject(
    'http://schooltool.org/ns/cambodia/levelgroups/group',
    'Group.',
    'The group to teach.')

LevelGroups = RelationshipSchema(URILevelGroups,
                                 level=URILevel,
                                 group=URILevelGroup)


URISectionEnrollment = URIObject(
    'http://schooltool.org/ns/cambodia/sectionenrollment/',
    'Groups enrolled to sections.',
    'Groups of people automatially enrolled to the sections.')

URISection = URIObject(
    'http://schooltool.org/ns/cambodia/sectionenrollment/section',
    'Section.',
    'The section that members of the group are enrolled into.')

SectionEnrollment = RelationshipSchema(URISectionEnrollment,
                                       section=URISection,
                                       group=URILevelGroup)

# For now, in the one and only current term:
# TODO: grade section DELETION when groups are unlinked from levels
#       the thing is... it's not doable without additional markup.
# TODO: grade section deletion when courses are unlinked with levels
# TODO: maybe: grade section deletion when groups are unlinked with levels


class CambodiaSectionCreatedEvent(object):
    def __init__(self, section):
        self.section = section


def createSection(term, course, group):
    section = Section(title=course.title)

    sections = ISectionContainer(term)
    namechooser = INameChooser(sections)
    name = namechooser.chooseName('', section)

    sections[name] = section

    section.courses.add(course)
    SectionEnrollment(section=section, group=group)
    notify(CambodiaSectionCreatedEvent(section))
    return section


def createTermSections(term, courses, groups):
    sections = []
    for course in courses:
        for group in groups:
            section = createSection(term, course, group)
            sections.append(section)
    return sections


def createSectionsForLevelGroups(event):
    """Add sections for level courses when groups are linked to levels."""
    if event.rel_type != URILevelGroups:
        return

    level = event[URILevel]
    group = event[URILevelGroup]
    courses = getRelatedObjects(
        level, URICourse, rel_type=URILevelCourses)

    schoolyear = ISchoolYear(level)
    for term in schoolyear.values():
        createTermSections(term, courses, [group])


def createSectionsForLevelCourses(event):
    """Add sections for groups when a course is linked to a level."""
    if event.rel_type != URILevelCourses:
        return

    level = event[URILevel]
    course = event[URICourse]
    groups = getRelatedObjects(
        level, URILevelGroup, rel_type=URILevelGroups)

    schoolyear = ISchoolYear(level)
    for term in schoolyear.values():
        createTermSections(term, [course], groups)


def enrollGroupsToCambodiaSection(event):
    """Manage enrollment of groups of students to Cambodian sections."""
    if event.rel_type != URISectionEnrollment:
        return

    section = event[URISection]
    group = event[URILevelGroup]

    if IRelationshipAddedEvent.providedBy(event):
        for student in group.members:
            if student not in section.memebers:
                section.members.add(student)
    elif IRelationshipRemovedEvent.providedBy(event):
        for student in group.members:
            if student in section.memebers:
                section.members.remove(student)


def enrollStudentsToCambodiaSections(event):
    """Manage enrollment of student to the group's Cambodian sections."""
    if event.rel_type != URIMembership:
        return

    student = event[URIMember]
    group = event[URIGroup]

    sections = getRelatedObjects(
        group, URISection, rel_type=URISectionEnrollment)

    for section in sections:
        if IRelationshipAddedEvent.providedBy(event):
            if student not in section.members:
                section.members.add(student)
        elif IRelationshipRemovedEvent.providedBy(event):
            if student in section.members:
                section.members.remove(student)


class PopulateSectionsForNewTerm(ObjectEventAdapterSubscriber):
    adapts(IObjectAddedEvent, ITerm)

    def __call__(self):
        term = self.object
        # XXX: The addIntIdSubscriber is a workaround.
        #      At the time of writing, adaptation to ISectionContainer
        #      from ITerm uses int ids.  Our code tries to adapt the
        #      newly created term to ISectionContainer and fails, because
        #      it obviously has no intid.   This should be fixed in
        #      the ISectionContainer(term) adapter.
        addIntIdSubscriber(term, ObjectAddedEvent(term))

        schoolyear = ISchoolYear(term)

        levels = interfaces.ILevelContainer(schoolyear)
        for level in levels.values():
            courses = getRelatedObjects(
                level, URICourse, rel_type=URILevelCourses)
            groups = getRelatedObjects(
                level, URILevelGroup, rel_type=URILevelGroups)
            if courses and groups:
                createTermSections(term, courses, groups)


def addClass(class_name, level):
    """Add a group that is linked to a level."""
    schoolyear = ISchoolYear(level)
    groups = IGroupContainer(schoolyear)

    group = Group(title=class_name, description=u'')
    namechooser = INameChooser(groups)
    name = namechooser.chooseName('', group)
    groups[name] = group

    LevelGroups(level=level, group=group)
    return group


DEFAULT_GRADES = [u'A', u'B', u'C']


class AddDefaultGroupsForNewLevel(ObjectEventAdapterSubscriber):
    adapts(IObjectAddedEvent, interfaces.ILevel)

    def __call__(self):
        level = self.object
        global DEFAULT_GRADES
        for grade_subtitle in DEFAULT_GRADES:
            addClass(u'%s%s' % (level.title, grade_subtitle), level)


class DeleteGroupsWhenLevelRemoved(ObjectEventAdapterSubscriber):
    adapts(IObjectRemovedEvent, interfaces.ILevel)

    def __call__(self):
        level = self.object
        groups = getRelatedObjects(
            level, URILevelGroup, rel_type=URILevelGroups)
        for group in groups:
            group_container = IGroupContainer(group)
            del group_container[group.__name__]
